@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2017 Klarälvdalens Datakonsult AB a KDAB Group company <info@kdab.com>
+    SPDX-FileCopyrightText: 2023 Harald Sitter <sitter@kde.org>
 
     Work sponsored by the LiMux project of the city of Munich.
     SPDX-FileContributor: Andras Mantia <andras.mantia@kdab.com>
@@ -20,8 +21,18 @@
 #include <Plasma/Corona>
 #include <chrono>
 
+#include "debug.h"
+
 using namespace std::chrono_literals;
 
+namespace
+{
+// The maximum amount of mappings we allow. This prevents performance and memory exhaustion problems when too many
+// items are on the desktop.
+// https://bugs.kde.org/show_bug.cgi?id=469445
+constexpr auto MAX_MAPPING_COUNT = 4096;
+} // namespace
+
 ScreenMapper *ScreenMapper::instance()
 {
     static ScreenMapper *s_instance = new ScreenMapper();
@@ -61,10 +72,9 @@ void ScreenMapper::removeScreen(int screenId, const QString &activity, const QUr
 
     const auto screenPathWithScheme = screenUrl.url();
     // store the original location for the items
-    auto it = m_screenItemMap.constBegin();
     QVector<QUrl> urlsToRemoveFromMapping;
-    while (it != m_screenItemMap.constEnd()) {
-        const auto name = it.key();
+    for (auto it = m_screenItemMap.constBegin(); it != m_screenItemMap.constEnd(); ++it) {
+        const auto &name = it.key();
         if (it.value() == screenId && name.first.url().startsWith(screenPathWithScheme) && name.second == activity) {
             bool found = false;
             for (const auto &disabledUrls : qAsConst(m_itemsOnDisabledScreensMap)) {
@@ -77,12 +87,11 @@ void ScreenMapper::removeScreen(int screenId, const QString &activity, const QUr
                 if (urlVectorIt == m_itemsOnDisabledScreensMap.end()) {
                     m_itemsOnDisabledScreensMap[pair] = {name.first};
                 } else {
-                    urlVectorIt->append(name.first);
+                    urlVectorIt->insert(name.first);
                 }
             }
             urlsToRemoveFromMapping.append(name.first);
         }
-        ++it;
     }
 
     saveDisabledScreensMap();
@@ -117,19 +126,21 @@ void ScreenMapper::addScreen(int screenId, const QString &activity, const QUrl &
     // restore the stored locations
     auto it = m_itemsOnDisabledScreensMap.find(pair);
     if (it != m_itemsOnDisabledScreensMap.end()) {
-        auto items = it.value();
-        for (const auto &name : it.value()) {
+        auto &items = it.value();
+        auto itemIt = items.begin();
+        while (itemIt != items.end()) {
             // add the items to the new screen, if they are on a disabled screen and their
             // location is below the new screen's path
-            if (name.url().startsWith(screenPathWithScheme)) {
-                addMapping(name, screenId, activity, DelayedSignal);
-                items.removeAll(name);
+            if (itemIt->url().startsWith(screenPathWithScheme)) {
+                addMapping(*itemIt, screenId, activity, DelayedSignal);
+                itemIt = items.erase(itemIt);
+                continue;
             }
+            itemIt = std::next(itemIt);
         }
-        if (items.isEmpty()) {
+
+        if (items.empty()) {
             m_itemsOnDisabledScreensMap.erase(it);
-        } else {
-            *it = items;
         }
     }
     saveDisabledScreensMap();
@@ -151,6 +162,18 @@ void ScreenMapper::addScreen(int screenId, const QString &activity, const QUrl &
 
 void ScreenMapper::addMapping(const QUrl &url, int screen, const QString &activity, MappingSignalBehavior behavior)
 {
+    if (m_screenItemMap.count() > MAX_MAPPING_COUNT) {
+        // Don't spam this
+        static auto reported = false;
+        if (!reported) {
+            qCCritical(FOLDER)
+                << "Greater than" << MAX_MAPPING_COUNT
+                << "files and folders on the desktop; this is too many to map their positions in a performant way! Not adding any more position mappings.";
+            reported = true;
+        }
+        return;
+    }
+
     m_screenItemMap[std::make_pair(url, activity)] = screen;
 
     if (behavior == DelayedSignal) {
@@ -193,7 +216,7 @@ void ScreenMapper::removeItemFromDisabledScreen(const QUrl &url)
 {
     for (auto it = m_itemsOnDisabledScreensMap.begin(); it != m_itemsOnDisabledScreensMap.end(); ++it) {
         auto urls = &(*it);
-        urls->removeAll(url);
+        urls->remove(url);
     }
 }
 
@@ -251,11 +274,19 @@ QStringList ScreenMapper::screenMapping() const
     QStringList result;
     result.reserve(m_screenItemMap.count() * 3); // Match setScreenMapping()
     auto it = m_screenItemMap.constBegin();
+    int i = 0;
     while (it != m_screenItemMap.constEnd()) {
+        if (i >= MAX_MAPPING_COUNT) {
+            qCCritical(FOLDER)
+                << "Greater than" << MAX_MAPPING_COUNT
+                << "disabled files and folders; this is too many to remember their position in a performant way! Not adding any more position mappings.";
+            break;
+        }
         result.append(it.key().first.toString());
         result.append(QString::number(it.value())); // Screen ID
         result.append(it.key().second); // Activity ID
         ++it;
+        ++i;
     }
 
     return result;
@@ -264,16 +295,17 @@ QStringList ScreenMapper::screenMapping() const
 void ScreenMapper::setScreenMapping(const QStringList &mapping)
 {
     decltype(m_screenItemMap) newMap;
-    const int count = mapping.count();
-    const bool useDefaultActivity = count % 3 != 0; // Missing activity ID from the old config before 5.25
-    const int sizeOfParamGroup = useDefaultActivity ? 2 : 3; // Match screenMapping()
+    // <url>,<screen id>,<activity id>
+    constexpr int sizeOfParamGroup = 3; // Match screenMapping()
+    const int count = std::min<int>(mapping.size(), MAX_MAPPING_COUNT * sizeOfParamGroup);
+    Q_ASSERT(count % sizeOfParamGroup == 0);
 
     newMap.reserve(count / sizeOfParamGroup);
     QMap<int, int> screenConsistencyMap;
     for (int i = 0; i < count - (sizeOfParamGroup - 1); i += sizeOfParamGroup) {
         if (i + (sizeOfParamGroup - 1) < count) {
             const QUrl url = QUrl::fromUserInput(mapping[i], {}, QUrl::AssumeLocalFile);
-            const QString activity = useDefaultActivity ? KActivities::Consumer().currentActivity() : mapping[i + 2];
+            const QString activity = mapping[i + 2];
             newMap[std::make_pair(url, activity)] = mapping[i + 1].toInt();
             screenConsistencyMap[mapping[i + 1].toInt()] = -1;
         }
@@ -313,11 +345,17 @@ QStringList ScreenMapper::disabledScreensMap() const
 {
     QStringList serializedMap;
     auto it = m_itemsOnDisabledScreensMap.constBegin();
-    for (; it != m_itemsOnDisabledScreensMap.constEnd(); ++it) {
+    for (int i = 0; it != m_itemsOnDisabledScreensMap.constEnd(); it = std::next(it), ++i) {
+        if (i >= MAX_MAPPING_COUNT) {
+            qCCritical(FOLDER)
+                << "Greater than" << MAX_MAPPING_COUNT
+                << "files and folders on the desktop; this is too many to map their positions in a performant way! Not adding any more position mappings.";
+            break;
+        }
         serializedMap.append(QString::number(it.key().first)); // Screen ID
         serializedMap.append(it.key().second); // Activity ID
         const auto urls = it.value();
-        serializedMap.append(QString::number(urls.size()));
+        serializedMap.append(QString::number(urls.size())); // Number of urls
         for (const auto &url : urls) {
             serializedMap.append(url.toString());
         }
@@ -336,21 +374,15 @@ void ScreenMapper::readDisabledScreensMap(const QStringList &serializedMap)
     QString activityId;
     int vectorCounter = 0;
 
-
+    // <screen id>,<activity id>,<number of urls>,<url 1>, ...
     for (const auto &entry : serializedMap) {
         if (readingScreenId) {
             screenId = entry.toInt();
             readingScreenId = false;
         } else if (readingActivityId) {
-            // Missing activity ID in the old config before 5.25
-            if (entry.toInt() > 0) {
-                vectorSize = entry.toInt();
-                activityId = KActivities::Consumer().currentActivity();
-            } else { // When a string is a uuid, toInt() will return 0
-                activityId = entry;
-            }
+            activityId = entry;
             readingActivityId = false;
-        } else if (vectorSize == -1) {
+        } else if (vectorSize == -1 /*number of urls is not read*/) {
             vectorSize = entry.toInt();
         } else {
             const auto url = stringToUrl(entry);
@@ -359,7 +391,7 @@ void ScreenMapper::readDisabledScreensMap(const QStringList &serializedMap)
             if (urlVectorIt == m_itemsOnDisabledScreensMap.end()) {
                 m_itemsOnDisabledScreensMap[pair] = {url};
             } else {
-                urlVectorIt->append(url);
+                urlVectorIt->insert(url);
             }
             vectorCounter++;
             if (vectorCounter == vectorSize) {
