@@ -9,6 +9,8 @@
 #include "componentchooserbrowser.h"
 #include "componentchooseremail.h"
 #include "componentchooserfilemanager.h"
+#include "componentchoosergeo.h"
+#include "componentchoosertel.h"
 #include "componentchooserterminal.h"
 
 ComponentChooserData::ComponentChooserData(QObject *parent, const QVariantList &args)
@@ -17,6 +19,8 @@ ComponentChooserData::ComponentChooserData(QObject *parent, const QVariantList &
     , m_fileManagers(new ComponentChooserFileManager(this))
     , m_terminalEmulators(new ComponentChooserTerminal(this))
     , m_emailClients(new ComponentChooserEmail(this))
+    , m_geoUriHandlers(new ComponentChooserGeo(this))
+    , m_telUriHandlers(new ComponentChooserTel(this))
 {
     load();
 }
@@ -27,6 +31,8 @@ void ComponentChooserData::load()
     m_fileManagers->load();
     m_terminalEmulators->load();
     m_emailClients->load();
+    m_geoUriHandlers->load();
+    m_telUriHandlers->load();
 }
 
 void ComponentChooserData::save()
@@ -35,6 +41,8 @@ void ComponentChooserData::save()
     m_fileManagers->save();
     m_terminalEmulators->save();
     m_emailClients->save();
+    m_geoUriHandlers->save();
+    m_telUriHandlers->save();
 }
 
 void ComponentChooserData::defaults()
@@ -43,16 +51,20 @@ void ComponentChooserData::defaults()
     m_fileManagers->defaults();
     m_terminalEmulators->defaults();
     m_emailClients->defaults();
+    m_geoUriHandlers->defaults();
+    m_telUriHandlers->defaults();
 }
 
 bool ComponentChooserData::isDefaults() const
 {
-    return m_browsers->isDefaults() && m_fileManagers->isDefaults() && m_terminalEmulators->isDefaults() && m_emailClients->isDefaults();
+    return m_browsers->isDefaults() && m_fileManagers->isDefaults() && m_terminalEmulators->isDefaults() && m_emailClients->isDefaults()
+        && m_geoUriHandlers->isDefaults() && m_telUriHandlers->isDefaults();
 }
 
 bool ComponentChooserData::isSaveNeeded() const
 {
-    return m_browsers->isSaveNeeded() || m_fileManagers->isSaveNeeded() || m_terminalEmulators->isSaveNeeded() || m_emailClients->isSaveNeeded();
+    return m_browsers->isSaveNeeded() || m_fileManagers->isSaveNeeded() || m_terminalEmulators->isSaveNeeded() || m_emailClients->isSaveNeeded()
+        || m_geoUriHandlers->isSaveNeeded() || m_telUriHandlers->isSaveNeeded();
 }
 
 ComponentChooser *ComponentChooserData::browsers() const
@@ -74,3 +86,13 @@ ComponentChooser *ComponentChooserData::emailClients() const
 {
     return m_emailClients;
 }
+
+ComponentChooser *ComponentChooserData::geoUriHandlers() const
+{
+    return m_geoUriHandlers;
+}
+
+ComponentChooser *ComponentChooserData::telUriHandlers() const
+{
+    return m_telUriHandlers;
+}
